#ifndef DISCORD_API_H
#define DISCORD_API_H

#include <Arduino.h>
#include <ESP8266HTTPClient.h>
#include <WiFiClient.h>
#include <WiFiClientSecureBearSSL.h>
#include <ESP8266WiFi.h>

class DiscordAPI{
public:
    /**
     * @brief Construct a new Discord API object
     * @param hookUrl url of the discord webhook
     */
    DiscordAPI(String hookUrl);

    /**
     * @brief send a message to the discord webhook
     * @param trame trame a envoyer
     * @return true la trame est bien envoyée
     * @return false la trame n'est pas envoyée
     */
    bool sendMessage(String trame);

    /**
     * @brief envoie un embed discord avec les horaires d'ouverture et de fermeture du lab
     *
     * @param hStart heure d'ouverture
     * @param hEnd heure de fermeture
     * @return true la trame est bien envoyée
     * @return false la trame n'est pas envoyée
     */
    bool sendHeure(String hStart, String hEnd);

private:

    /**
     * @brief webhook URL
     */
    String hookUrl;

    /**
     * @brief http client
     */
    HTTPClient* httpClient;

    /**
     * @brief wifi client (for https)
     */
    BearSSL::WiFiClientSecure* wifiClient;
};


#endif //DISCORD_API_H
